@@ -110,12 +110,13 @@ from .history import History
 from .hist_persist import HistPersist
 from .autocomplete import Autocomplete
 from .call_tips import CallTips
+from .autoparen import Autoparen
 from .subprocess_handler import SubprocessHandler, StartError
 from .beep import beep
 from .file_dialogs import save_dialog
 from .tags import (OUTPUT, STDIN, STDOUT, STDERR, EXCEPTION, PROMPT, COMMAND,
                    COMMAND_DEFS, COMMAND_SEP, MESSAGE, RESULT_IND, RESULT)
-import tags
+from . import tags
 
 INDENT_WIDTH = 4
 
@@ -141,6 +142,7 @@ class DreamPie(SimpleGladeApp):
     def __init__(self, pyexec):
         SimpleGladeApp.__init__(self, gladefile, 'window_main')
         self.load_popup_menus()
+        self.set_mac_accelerators()
         
         self.config = Config()
 
@@ -155,9 +157,6 @@ class DreamPie(SimpleGladeApp):
 
         self.init_sourcebufferview()
         
-        # List of function names which expect a string argument
-        self.expects_str = []
-        
         self.configure()
 
         self.output = Output(self.textview)
@@ -198,6 +197,12 @@ class DreamPie(SimpleGladeApp):
 
         self.call_tips = CallTips(self.sourceview, self.get_func_doc,
                                   INDENT_WIDTH)
+        
+        self.autoparen = Autoparen(self.sourcebuffer,
+                                   self.is_callable_only,
+                                   self.get_expects_str,
+                                   self.autoparen_show_call_tip,
+                                   INDENT_WIDTH)
 
         self.subp = SubprocessHandler(
             pyexec, data_dir,
@@ -233,7 +238,7 @@ class DreamPie(SimpleGladeApp):
             self.show_getting_started_dialog()
             self.config.set_bool('show-getting-started', False)
             self.config.save()
-
+        
     def load_popup_menus(self):
         # Load popup menus from the glade file. Would not have been needed if
         # popup menus could be children of windows.
@@ -248,6 +253,40 @@ class DreamPie(SimpleGladeApp):
         self.copy_section_menu = xml.get_widget('copy_section_menu')
         self.view_section_menu = xml.get_widget('view_section_menu')
         self.save_section_menu = xml.get_widget('save_section_menu')
+    
+    def set_mac_accelerators(self):
+        # Set up accelerators suitable for the Mac.
+        # Ctrl-Up and Ctrl-Down are taken by the window manager, so we use
+        # Ctrl-PgUp and Ctrl-PgDn.
+        # We want it to be easy to switch, so both sets of keys are always
+        # active, but only one, most suitable for each platform, is displayed
+        # in the menu.
+        
+        accel_group = gtk.accel_groups_from_object(self.window_main)[0]
+        menu_up = self.menuitem_history_up
+        UP = gdk.keyval_from_name('Up')
+        PGUP = gdk.keyval_from_name('Prior')
+        menu_dn = self.menuitem_history_down
+        DN = gdk.keyval_from_name('Down')
+        PGDN = gdk.keyval_from_name('Next')
+
+        if sys.platform != 'darwin':
+            menu_up.add_accelerator('activate', accel_group, PGUP,
+                                    gdk.CONTROL_MASK, 0)
+            menu_dn.add_accelerator('activate', accel_group, PGDN,
+                                    gdk.CONTROL_MASK, 0)
+        else:
+            menu_up.remove_accelerator(accel_group, UP, gdk.CONTROL_MASK)
+            menu_up.add_accelerator('activate', accel_group, PGUP,
+                                    gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+            menu_up.add_accelerator('activate', accel_group, UP,
+                                    gdk.CONTROL_MASK, 0)
+    
+            menu_dn.remove_accelerator(accel_group, DN, gdk.CONTROL_MASK)
+            menu_dn.add_accelerator('activate', accel_group, PGDN,
+                                    gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+            menu_dn.add_accelerator('activate', accel_group, DN,
+                                    gdk.CONTROL_MASK, 0)
 
     def on_cut(self, _widget):
         return self.selection.cut()
@@ -487,47 +526,6 @@ class DreamPie(SimpleGladeApp):
 
         return False
 
-    @sourceview_keyhandler('space', 0)
-    def on_sourceview_space(self):
-        """
-        If a space was hit after a callable-only object, add parentheses.
-        """
-        if self.is_executing:
-            return False
-        
-        sb = self.sourcebuffer
-        insert = sb.get_iter_at_mark(sb.get_insert())
-        insert_linestart = sb.get_iter_at_line(insert.get_line())
-        line = self.sb_get_text(insert_linestart, insert)
-        # Search for a sequence of identifiers separated by dots at the
-        # end of the line
-        m = re.search(r'[a-zA-Z_][a-zA-Z0-9_]*(?:\.[a-zA-Z_][a-zA-Z0-9_]*)*$',
-                      line)
-        if not m:
-            return False
-        what = m.group()
-        if iskeyword(what):
-            # This is a common case, no need to ask the subprocess for it.
-            return False
-        
-        is_callable_only, expects_str = self.call_subp(u'is_callable_only', what)
-        if not is_callable_only:
-            return False
-        
-        last_name = what.rsplit('.', 1)[-1]
-        if expects_str or last_name in self.expects_str:
-            sb.insert(insert, '("")')
-            insert.backward_chars(2)
-        else:
-            sb.insert(insert, '()')
-            insert.backward_char()
-        sb.place_cursor(insert)
-        
-        # Show a call tip
-        self.call_tips.show(is_auto=True)
-        
-        return True
-        
     # The following 3 handlers are for characters which may trigger automatic
     # opening of the completion list. (slash and backslash depend on path.sep)
     # We leave the final decision whether to open the list to the autocompleter.
@@ -566,6 +564,33 @@ class DreamPie(SimpleGladeApp):
     def on_sourceview_keypress(self, _widget, event):
         return handle_keypress(self, event, sourceview_keyhandlers)
 
+
+    # Autoparen
+    
+    @sourceview_keyhandler('space', 0)
+    def on_sourceview_space(self):
+        """
+        If a space was hit after a callable-only object, add parentheses.
+        """
+        if self.is_executing:
+            return False
+        if not self.config.get_bool('autoparen'):
+            return False
+        
+        return self.autoparen.add_parens()
+
+    def is_callable_only(self, expr):
+        # This should be called only as a result of on_sourceview_space, which
+        # already checks that is_executing==False.
+        return self.call_subp(u'is_callable_only', expr)
+    
+    def get_expects_str(self):
+        return set(self.config.get('expects-str-2').split())
+    
+    def autoparen_show_call_tip(self):
+        self.call_tips.show(is_auto=True)
+
+
     # History
 
     def on_textview_keypress(self, _widget, event):
@@ -599,7 +624,8 @@ class DreamPie(SimpleGladeApp):
         
         self.call_subp(u'set_pprint', config.get_bool('pprint'))
         
-        self.call_subp(u'set_matplotlib_ia_warn',
+        self.call_subp(u'set_matplotlib_ia',
+                       config.get_bool('matplotlib-ia-switch'),
                        config.get_bool('matplotlib-ia-warn'))
         
     def run_init_code(self):
@@ -973,8 +999,6 @@ class DreamPie(SimpleGladeApp):
         command_defs = self.textbuffer.get_tag_table().lookup(COMMAND_DEFS)
         command_defs.props.invisible = config.get_bool('hide-defs')
 
-        self.expects_str = set(eval(config.get('expects-str')).split())
-
     def on_preferences(self, _widget):
         cd = ConfigDialog(self.config, gladefile, self.window_main)
         r = cd.run()
