@@ -58,13 +58,21 @@ class ConfigDialog(SimpleGladeApp):
         
         self.viewer_entry.props.text = eval(config.get('viewer'))
         
-        self.expects_str_entry.props.text = eval(config.get('expects-str'))
+        self.autoparen_chk.props.active = config.get_bool('autoparen')
+        self.expects_str_entry.props.text = config.get('expects-str-2')
         
         self.leave_code_chk.props.active = config.get_bool('leave-code')
         
         self.hide_defs_chk.props.active = config.get_bool('hide-defs')
         
-        self.matplotlib_ia_warn_chk.props.active = config.get_bool('matplotlib-ia-warn')
+        switch = config.get_bool('matplotlib-ia-switch')
+        warn = config.get_bool('matplotlib-ia-warn')
+        if switch:
+            self.matplotlib_ia_switch_rad.props.active = True
+        elif warn:
+            self.matplotlib_ia_warn_rad.props.active = True
+        else:
+            self.matplotlib_ia_ignore_rad.props.active = True
     
         self.ask_on_quit_chk.props.active = config.get_bool('ask-on-quit')
     
@@ -138,13 +146,23 @@ class ConfigDialog(SimpleGladeApp):
         
         config.set('viewer', repr(self.viewer_entry.props.text.decode('utf8').strip()))
         
-        config.set('expects-str', repr(expects_str))
+        config.set_bool('autoparen', self.autoparen_chk.props.active)
+        config.set('expects-str-2', expects_str)
         
         config.set_bool('leave-code', self.leave_code_chk.props.active)
         
         config.set_bool('hide-defs', self.hide_defs_chk.props.active)
         
-        config.set_bool('matplotlib-ia-warn', self.matplotlib_ia_warn_chk.props.active)
+        if self.matplotlib_ia_switch_rad.props.active:
+            switch = True
+            warn = False
+        elif self.matplotlib_ia_warn_rad.props.active:
+            switch = False
+            warn = True
+        else:
+            switch = warn = False
+        config.set_bool('matplotlib-ia-switch', switch)
+        config.set_bool('matplotlib-ia-warn', warn)
         
         config.set_bool('ask-on-quit', self.ask_on_quit_chk.props.active)
         
@@ -427,4 +445,5 @@ class ConfigDialog(SimpleGladeApp):
     def on_autofold_chk_toggled(self, _widget):
         self.autofold_spin.props.sensitive = self.autofold_chk.props.active
 
-
+    def on_autoparen_chk_toggled(self, _widget):
+        self.expects_str_alignment.props.sensitive = self.autoparen_chk.props.active
