@@ -15,6 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with DreamPie.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import with_statement
 import sys
 py3k = (sys.version_info[0] == 3)
 import os
@@ -31,6 +32,9 @@ import inspect
 import pydoc
 import pprint
 import codeop
+import signal
+from contextlib import contextmanager
+from itertools import chain
 try:
     # Executing multiple statements in 'single' mode (print results) is done
     # with the ast module. Python 2.5 doesn't have it, so we use the compiler
@@ -39,6 +43,14 @@ try:
     import ast
 except ImportError:
     ast = None
+else:
+    # IronPython 2.7.1 has the ast module, but can't compile from an AST.
+    # If that's the case, the ast module doesn't interest us.
+    try:
+        compile(compile('a', 'fn', 'exec', ast.PyCF_ONLY_AST), 'fn', 'exec')
+    except TypeError:
+        ast = None
+if ast is None:
     from .split_to_singles import split_to_singles
 import __future__
 
@@ -61,6 +73,9 @@ from logging import debug
 # time interval to process GUI events, in seconds
 GUI_SLEEP = 0.1
 
+# Maximum result string length to transmit
+MAX_RES_STR_LEN = 1000000
+
 rpc_funcs = set()
 # A decorator which adds the function name to rpc_funcs
 def rpc_func(func):
@@ -95,11 +110,90 @@ operator_methods = ['__%s__' % s for s in
                     'add sub mul div floordiv truediv mod divmod pow lshift '
                     'rshift and xor or'.split()]
 
+quit_msg = """\
+Press Ctrl-Q or close the window if you want to quit DreamPie.
+Press Ctrl-F6 if you want to restart the subprocess."""
+class Quit(object):
+    def __repr__(self):
+        return quit_msg
+    def __call__(self):
+        raise RuntimeError(quit_msg)
+
+_simple_types = (
+    bool, int, float, complex, type(None), slice,
+    long, # 2to3 replaces long with int, so this is fine
+    )
+_string_types = (
+    bytes if py3k else str, # 2to3 can't replace str with bytes
+    unicode, # 2to3 replaces unicode with str, so this is fine
+    )
+def is_key_reprable(obj, max_depth=2):
+    """
+    Check whether an object (which is a dict key) simple enough
+    to be used in the completion list.
+    This checks that it's of simple types, and has some arbitrary limits.
+    """
+    if type(obj) in _simple_types:
+        return True
+    elif type(obj) in _string_types:
+        return len(obj) < 1000
+    elif type(obj) in (tuple, frozenset):
+        if max_depth <= 0 or len(obj) > 5:
+            return False
+        else:
+            return all(is_key_reprable(x, max_depth-1) for x in obj)
+    else:
+        return False
+
+# SIGINT masking
+
+def can_mask_sigint():
+    return sys.platform in ('linux2', 'win32')
+
+def unmask_sigint():
+    # We want to mask ctrl-c events when not running user code, to allow
+    # the main process to send a SIGINT anytime, in order to allow it to
+    # break GUI code executed when idle.
+    # If we can't mask ctrl-c events, the main process will only send
+    # SIGINT when executing user code (it uses the get_subprocess_info
+    # method to know that.)
+    # Also, on win32 we may get unwanted ctrl-c events if the user is
+    # running a subprocess. Since all processes in the same "console group"
+    # get the event, the subprocess may get it before us and exit. Then we 
+    # get out of the try-except block, and only later we get the ctrl-c.
+    if sys.platform == 'linux2':
+        signal.signal(signal.SIGINT, signal.default_int_handler)
+    elif sys.platform == 'win32':
+        windll.kernel32.SetConsoleCtrlHandler(None, False)
+    else:
+        pass
+
+def mask_sigint():
+    if sys.platform == 'linux2':
+        signal.signal(signal.SIGINT, signal.SIG_IGN)
+    elif sys.platform == 'win32':
+        windll.kernel32.SetConsoleCtrlHandler(None, True)
+    else:
+        pass
+
+@contextmanager
+def unmasked_sigint():
+    try:
+        unmask_sigint()
+        yield
+    finally:
+        mask_sigint()
+
+
+
 class Subprocess(object):
     def __init__(self, port):
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.sock.connect(('localhost', port))
 
+        # Mask SIGINT/Ctrl-C
+        mask_sigint()
+        
         # Make sys.displayhook change self.last_res
         self.last_res = None
         sys.displayhook = self.displayhook
@@ -117,21 +211,23 @@ class Subprocess(object):
         # (Otherwise multiprocessing on win32 starts running subp_main.py)
         sys.argv = ['']
 
-        # Remove __builtin__.exit, which only confuses users
-        del __builtin__.exit
+        # Adjust exit and quit objects
+        __builtin__.exit = __builtin__.quit = Quit()
         
         self.gui_handlers = [GtkHandler(), Qt4Handler(), TkHandler()]
+        self.idle_paused = False
 
         self.gid = 0
         self.flags = 0
         
         # Config
         self.is_pprint = False
+        self.is_matplotlib_ia_switch = False
         self.is_matplotlib_ia_warn = False
         self.reshist_size = 0
         
-        # Did we already warn about matplotlib non-interactive mode?
-        self.matplotlib_ia_warned = False
+        # Did we already handle matplotlib in non-interactive mode?
+        self.matplotlib_ia_handled = False
         
         # The result history index of the next value to enter the history
         self.reshist_counter = 0
@@ -141,7 +237,8 @@ class Subprocess(object):
 
     def loop(self):
         while True:
-            self.handle_gui_events(self.sock)
+            if not self.idle_paused:
+                self.handle_gui_events(self.sock)
             funcname, args = recv_object(self.sock)
             if funcname in rpc_funcs:
                 func = getattr(self, funcname)
@@ -158,7 +255,7 @@ class Subprocess(object):
                     send_object(self.sock, None)
             else:
                 # aid in debug
-                print >> sys.stderr, "Unknown command: %s" % funcname
+                sys.stderr.write("Unknown command: %s\n" % funcname)
                 send_object(self.sock, None)
 
     def displayhook(self, res):
@@ -279,7 +376,7 @@ class Subprocess(object):
             # which had no effect
             filename = '<pyshell#%d>' % self.gid
             self.gid += 1
-            lines = src.split("\n")
+            lines = [x+'\n' for x in src.split("\n")]
             linecache.cache[filename] = len(src)+1, None, lines, filename
             codeob = compile(src, filename, 'single', self.flags)
             self.flags = self.update_features(self.flags, codeob.co_flags)
@@ -287,6 +384,20 @@ class Subprocess(object):
         
         return True, codeobs
     
+    @staticmethod
+    def safe_pformat(obj):
+        """
+        Use pprint to format an object.
+        In case of an exception, warn and use regular repr instead.
+        """
+        try:
+            return unicode(pprint.pformat(obj))
+        except:
+            from warnings import warn
+            warn('pprint raised an exception, using repr instead. '
+                 'To reproduce, run: "from pprint import pprint; pprint(_)"')
+            return unicode(repr(obj))
+    
     @rpc_func
     def execute(self, source):
         """
@@ -302,6 +413,9 @@ class Subprocess(object):
         exception_string - description of the exception, or None if is_success.
         rem_stdin - data that was sent into stdin and wasn't consumed.
         """
+        # pause_idle was called before execute, disable it.
+        self.idle_paused = False
+        
         if ast:
             success, r = self.compile_ast(source)
         else:
@@ -315,23 +429,28 @@ class Subprocess(object):
             
         self.last_res = None
         try:
-            # Execute
-            for codeob in codeobs:
-                exec codeob in self.locs
-            # Work around http://bugs.python.org/issue8213 - stdout buffered
-            # in Python 3.
-            sys.stdout.flush()
-            sys.stderr.flush()
-            # Convert the result to a string. This is here because exceptions
-            # may be raised here.
-            if self.last_res is not None:
-                if self.is_pprint:
-                    res_str = unicode(pprint.pformat(self.last_res))
+            with unmasked_sigint():
+                # Execute
+                for codeob in codeobs:
+                    exec codeob in self.locs
+                # Work around http://bugs.python.org/issue8213 - stdout buffered
+                # in Python 3.
+                sys.stdout.flush()
+                sys.stderr.flush()
+                # Convert the result to a string. This is here because exceptions
+                # may be raised here.
+                if self.last_res is not None:
+                    if self.is_pprint:
+                        res_str = self.safe_pformat(self.last_res)
+                    else:
+                        res_str = unicode(repr(self.last_res))
+                    if len(res_str) > MAX_RES_STR_LEN:
+                        res_str = (res_str[:MAX_RES_STR_LEN]
+                                   +'\n[%d chars truncated]' % (
+                                        len(res_str)-MAX_RES_STR_LEN))
                 else:
-                    res_str = unicode(repr(self.last_res))
-            else:
-                res_str = None
-        except (Exception, KeyboardInterrupt):
+                    res_str = None
+        except:
             sys.stdout.flush()
             excinfo = sys.exc_info()
             sys.last_type, sys.last_value, sys.last_traceback = excinfo
@@ -357,7 +476,7 @@ class Subprocess(object):
                 if not r:
                     # File may be in error state
                     break
-                rem_stdin.append(r)
+                rem_stdin.append(unicodify(r))
         elif sys.platform == 'win32':
             fd = sys.stdin.fileno()
             handle = get_osfhandle(fd)
@@ -377,13 +496,27 @@ class Subprocess(object):
 
         yield is_success, res_no, res_str, exception_string, rem_stdin
 
+    @rpc_func
+    def pause_idle(self):
+        """
+        before 'execute' is called, 'pause_idle' is called to check if we don't
+        do any idle jobs right now. It is followed by either 'execute' or
+        'resume_idle'.
+        """
+        self.idle_paused = True
+    
+    @rpc_func
+    def resume_idle(self):
+        self.idle_paused = False
+    
     @rpc_func
     def set_pprint(self, is_pprint):
         self.is_pprint = is_pprint
     
     @rpc_func
-    def set_matplotlib_ia_warn(self, is_matplotlib_ia_warn):
-        self.is_matplotlib_ia_warn = is_matplotlib_ia_warn
+    def set_matplotlib_ia(self, is_switch, is_warn):
+        self.is_matplotlib_ia_switch = is_switch
+        self.is_matplotlib_ia_warn = is_warn
 
     @rpc_func
     def set_reshist_size(self, new_reshist_size):
@@ -495,13 +628,31 @@ class Subprocess(object):
         except Exception:
             return None
         try:
-            args = inspect.getargspec(obj)[0]
+            if not py3k:
+                args = inspect.getargspec(obj)[0]
+            else:
+                args = inspect.getfullargspec(obj).args
         except TypeError:
             return None
         # There may be nested args, so we filter them
         return [unicodify(s) for s in args
                 if isinstance(s, basestring)]
-    
+
+    @rpc_func
+    def complete_dict_keys(self, expr):
+        """
+        Return the reprs of the dict's keys (a list of strings)
+        Returns only those which have a simple-enough repr.
+        """
+        try:
+            obj = eval(expr, self.locs)
+        except Exception:
+            return None
+        if not isinstance(obj, dict) or len(obj) > 1000:
+            return None
+        return sorted(unicodify(repr(x)) for x in obj if is_key_reprable(x))
+        
+
     @rpc_func
     def find_modules(self, package):
         if package:
@@ -598,8 +749,8 @@ class Subprocess(object):
         
         return public, private, case_insen_filenames
     
-    @rpc_func
-    def get_welcome(self):
+    @staticmethod
+    def get_welcome():
         if 'IronPython' in sys.version:
             first_line = sys.version[sys.version.find('(')+1:sys.version.rfind(')')]
         else:
@@ -611,6 +762,10 @@ class Subprocess(object):
         return (first_line+'\n'
                 +u'Type "copyright", "credits" or "license()" for more information.\n')
 
+    @rpc_func
+    def get_subprocess_info(self):
+        return (self.get_welcome(), can_mask_sigint())
+        
     @classmethod
     def _find_constructor(cls, class_ob):
         # Given a class object, return a function object used for the
@@ -630,14 +785,39 @@ class Subprocess(object):
             obj = eval(expr, self.locs)
         except Exception:
             return None
+        if isinstance(obj, (types.BuiltinFunctionType,
+                            types.BuiltinMethodType)):
+            # These don't have source code, and using pydoc will only
+            # add something like "execfile(...)" before the doc.
+            doc = inspect.getdoc(obj)
+            if doc is None:
+                return None
+            return unicodify(doc)
+        
+        # Check if obj.__doc__ is not in the code (was added after definition).
+        # If so, return pydoc's documentation.
+        # This test is CPython-specific. Another approach would be to look for
+        # the string in the source code.
+        co_consts = getattr(getattr(obj, 'func_code', None), 'co_consts', None)
+        __doc__ = getattr(obj, '__doc__', None)
+        if co_consts is not None and __doc__ is not None:
+            if __doc__ not in co_consts:
+                # Return pydoc's documentation
+                return unicodify(textdoc.document(obj).strip())
+        
         try:
             source = inspect.getsource(obj)
         except (TypeError, IOError):
+            # If can't get the source, return pydoc's documentation
             return unicodify(textdoc.document(obj).strip())
         else:
+            # If we can get the source, return it.
+            
             # cleandoc removes extra indentation.
             # We add a newline because it ignores indentation of first line...
-            return unicodify(inspect.cleandoc('\n'+source))
+            # The next line is for Python 2.5 compatibility.
+            cleandoc = getattr(inspect, 'cleandoc', lambda s: s)
+            return unicodify(cleandoc('\n'+source))
     
     @rpc_func
     def is_callable_only(self, what):
@@ -670,30 +850,30 @@ class Subprocess(object):
         return r, expects_str
     
     def check_matplotlib_ia(self):
-        """Warn if matplotlib is in non-interactive mode"""
-        if not self.is_matplotlib_ia_warn:
+        """Check if matplotlib is in non-interactive mode, and handle it."""
+        if not self.is_matplotlib_ia_warn and not self.is_matplotlib_ia_switch:
             return
-        if self.matplotlib_ia_warned:
+        if self.matplotlib_ia_handled:
             return
         if 'matplotlib' not in sys.modules:
             return
-        self.matplotlib_ia_warned = True
+        self.matplotlib_ia_handled = True
         # From here we do this only once.
         matplotlib = sys.modules['matplotlib']
-        if not hasattr(matplotlib, 'rcParams'):
+        if not hasattr(matplotlib, 'is_interactive'):
             return
-        rcParams = matplotlib.rcParams
-        if not isinstance(rcParams, dict):
+        if matplotlib.is_interactive():
             return
-        if 'interactive' not in rcParams:
-            return
-        if not rcParams['interactive']:
+        if self.is_matplotlib_ia_switch:
+            if not hasattr(matplotlib, 'interactive'):
+                return
+            matplotlib.interactive(True)
+        else:
             sys.stderr.write(
                 "Warning: matplotlib in non-interactive mode detected.\n"
-                "This will mean plots will appear only after you run show().\n"
-                "To fix this, set \"interactive : True\" in your matplotlibrc file.\n"
-                "To suppress this warning, disable the matplotlib warning in the\n"
-                "configuration window.\n")
+                "This means that plots will appear only after you run show().\n"
+                "Use Edit->Preferences->Shell to automatically switch to interactive mode \n"
+                "or to suppress this warning.\n")
 
 
 # Handle GUI events
@@ -708,6 +888,20 @@ class GuiHandler(object):
         """
         raise NotImplementedError("Abstract method")
 
+@contextmanager
+def user_code():
+    """
+    Run user code, unmasking SIGINT, and catching exceptions.
+    """
+    try:
+        unmask_sigint()
+        yield
+    except:
+        sys.excepthook(*sys.exc_info())
+    finally:
+        mask_sigint()
+
+
 class GtkHandler(GuiHandler):
     def __init__(self):
         self.gtk = None
@@ -725,7 +919,9 @@ class GtkHandler(GuiHandler):
             else:
                 return False
         self.timeout_add(int(delay * 1000), self.gtk_main_quit)
-        self.gtk.main()
+        with user_code():
+            self.gtk.main()
+            
         return True
 
     def gtk_main_quit(self):
@@ -757,7 +953,8 @@ class Qt4Handler(GuiHandler):
         QtCore.QObject.connect(timer, QtCore.SIGNAL('timeout()'),
                                self.qt4_quit_if_no_modal)
         timer.start(delay*1000)
-        self.app.exec_()
+        with user_code():
+            self.app.exec_()
         timer.stop()
         QtCore.QObject.disconnect(timer, QtCore.SIGNAL('timeout()'),
                                   self.qt4_quit_if_no_modal)
@@ -790,8 +987,9 @@ class TkHandler(GuiHandler):
         # is something before we handle Tk events.
         if Tkinter._default_root:
             _tkinter = Tkinter._tkinter
-            while _tkinter.dooneevent(_tkinter.DONT_WAIT):
-                pass
+            with user_code():
+                while _tkinter.dooneevent(_tkinter.DONT_WAIT):
+                    pass
         time.sleep(delay)
         return True
 
